****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: finalizeptsmasterlist.sas  
*
* Created (mm/dd/yyyy): 02/02/2017
* Last modified: 11/20/2017
* Version: 1.4
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This program will finalize the numerator dataset by doing final calculation and variables assessment 
*
*  Program inputs:                                                                                   
*	-Dataset with valid numerator observations (_PtsMasterList)
* 
*  Program outputs:                                                                                                                                       
* 	-Dataset with valid numerator observations (_PtsMasterList)
* 
* PARAMETERS:
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1		  05/04/17		AP		Added race_out and hispanic_out cohortfile parameters
*
*   1.2		  10/12/17		AP		Modified to only include age at index date in Type 5 (QRP-446)
*
*	1.3	 	  10/12/17		AP		Added functionality to keep blackout and updated Type 4 PrePostInd 
*									(QRP-319, QRP-388)
*
*	1.4	 	  11/20/17		AP		REmoved functionality to keep blackout (QRP-466) 
*
***************************************************************************************************;

%macro ms_finalizeptsmasterlist();
	
%put =====> MACRO CALLED: ms_finalizeptsmasterlist v1.4;

	data _PtsMasterList;
	set _PtsMasterList;
	format Group $30.;
    Group="&ITGROUP."; 
    Year=Year(Indexdt);
    Type=input("&type.",best.);
	run;

	****************************************;
	* Type 1 or 2 cohort definition 		;
	****************************************;
	%if %eval(&type=1 | &type=2) %then %do;

    	proc datasets library=work nolist nowarn;
    	delete _GroupUtilFile;
    	quit;

		%if %eval(&type=1) %then %do; *TYPE 1 ONLY;

		
		/*evaluate enrolldays (>=1 prior to or equal to index date) - for mom-baby linkage*/
		%if %eval(&type.=1) %then %do;
			proc sql noprint;
				create table _indexenroll as
				select pts.patid,
					   pts.indexdt,
					   age.birth_date,
					   enr.enr_start,
					   enr.enr_end
				from _PtsMasterList as pts,
				enr_&group. as enr,
				_AgeGroups as age
				where pts.patid = enr.patid and pts.patid = age.patid and enr.enr_start <= pts.indexdt; 
			quit;

			data _indexenroll;
				set _indexenroll;

				orig_enr_start = enr_start;
				orig_enr_end = enr_end;

				enr_start = max(birth_date,enr_start);
				enr_end=(min(enr_end, indexdt));

				length_enrollment = enr_end-enr_start+1;

				if length_enrollment <1 then delete;
			run;

			proc sql noprint;
				create table enrolldays as 
				select patid,
					   indexdt,
					   sum(length_enrollment) as numenrolldays,
					   min(orig_enr_start) as cenr_start format=date9.
				from _indexenroll
				group by patid, indexdt
				order by patid, indexdt;
			quit;

			%let enrolldayscat1 = %sysfunc(tranwrd(&enrolldayscat., +, -999999999));
			%let numcat= %sysfunc(countw(&enrolldayscat1., ' '));
			%put &enrolldayscat1. &numcat.;

			data _PtsMasterList;
				length enrolldays $10;
				merge _PtsMasterList 
					  enrolldays;
				by patid indexdt;

				if numenrolldays <1 then delete;

				do i=1 to &numcat.;        
					if numenrolldays <= scan(scan("&enrolldayscat1.",i,' '),2, '-') then do;
				    	enrolldays = scan("&enrolldayscat.",i,' ');
						enrolldayssort = i;
				       	leave;
				    end;
				end;
				drop i;
			run;
			
		%end;

	        data _PtsMasterList;
	        set _PtsMasterList;
	        by Patid IndexDt;
	        if T1CohortDef ='01' then do;
	            if first.PatId;
	        end;

			FEventDt = .;
				
	        keep Group Type PatId Enr_start Enr_End year IndexDt EpisodeEndDt indexlook RxAmt RxSup NumDispensing FEventDt enrolldays: cenr_start;
	        run;

			* Because Type 1 analysis does not create episodes, the same index date can exist within a group
		  	if dispensings with different StockGroups have the same RXDate. Because episodes are not created, overlapping 
		  	can also occurs between claims that belong to different StockGroups. In this case the ms_shaveoutside macro
		  	can create duplicates and is not reliable. In Type 1, because each observation in _PtsMasterList represents
		  	a single claim, we can adjust RxSup and RxAmt using this dataset instead of using _GroupIndex;
			data _GroupUtilFile;
			set _PtsMasterList;		
			%if "&agedatecohort" = "N" %then %do;	
			TotRxSup=EpisodeEndDt-IndexDt+1;
			TotRxAmt=RxAmt*TotRxSup/RxSup;
			NumDisp=NumDispensing;	
			%end;
			%if "&agedatecohort" = "Y" %then %do;	
			TotRxSup=0;
			TotRxAmt=0;
			NumDisp=0;	
			%end;
			keep PatId IndexDt NumDisp TotRxSup TotRxAmt;
			run;
		%end;

		%else %do; *TYPE 2 ONLY;
			*Set first event date among all episodes for patients;
	        data _PtsMasterList;
	        merge _PtsMasterList
	              _FirstEventDtInEpisode;
	        by PatId;
	        run;

			*Set events and final variables;
			data _PtsMasterList;
	        merge _PtsMasterList(in=a)
	              _EPisodesWithEvents(in=b);
	        by Patid IndexDt;
	        if a;

	        tte=Min(EpisodeEndDt,Enr_End,FEventDt)-IndexDt -&BLACKOUTPER. +1;

	        if T2CohortDef ='01' then do;
	            if first.PatId;
	        end;
	        if T2CohortDef ='03' then do;
	            if FEventDt_Any and IndexDt > FEventDt_Any then delete;
	        end;	        

	        if &itt.=1 then EpisodeType="ITT";
	        else EpisodeType="EPI";

			/*Enumerate censoring criteria*/

			/*initialize censoring variables*/
			cens_elig = 0;
			cens_dth = 0;
			cens_dpend = 0;
			cens_qryend = 0;
			cens_episend = 0;	
			cens_spec = 0;
			cens_event = 0;

			/*Episode is not adjusted for events, so need to consider true censoring date*/
			EpisodeEndDt_Censor = min(EpisodeEndDt,Enr_End,FEventDt, deathdt);

			/*Censored due to event*/
			if EpisodeEndDt_Censor = FEventDt then do;
				Event_flag = 'Y';
				cens_event = 1;
			end;
			else do;
				Event_flag = 'N';
				cens_event = 0;
			end;

			/*Censored due to enrollment*/
			If EpisodeEndDt_Censor = Enr_End then cens_elig = 1; else cens_elig = 0;
				/*Reset if episode truncated due to death and original enrollment did not occur on death date*/
				if cens_elig = 1 and EnrEpisodeCensoredAtDeath = 1 and enr_end_equal_death ne 1 then cens_elig = 0;

			/*Censored due to death*/
			If EnrEpisodeCensoredAtDeath = 1 and EpisodeEndDt_Censor = Deathdt then cens_dth = 1; else cens_dth = 0;

			/*Censored due to query end*/
			if EpisodeEndDt_Censor = &enddate. then cens_qryend = 1; else cens_qryend = 0;

			/*Censored due to DP Max*/
			%if &Censor_DPEnd. = 1 %then %do;
				if EpisodeEndDt_Censor = &dp_maxdate. then cens_dpend = 1; else cens_dpend = 0;

					%if &DPMax_Adjust. =1 %then %do;
						Cens_QRYEND = 0; /*Reset Cens_qryend because we adjusted enddate to equal dp_maxdate*/
					%end;
			%end;

			/*Censored due to requester defined criteria*/
			if trunkdate and EpisodeEndDt_Censor = trunkdate then cens_spec = 1; else cens_spec = 0;

			/*Censored due to episode end date*/
				/*No adjustments have been made*/
				if (OrigEpisEndDt = EpisodeEndDt_Censor) then cens_episend = 1;
				else if not missing(maxepisdur) and EpisodeEndDt_Censor = IndexDt + maxepisdur -1 then cens_episend = 1;
				else cens_episend = 0;

			/*Adjust enr_end for death (for denoms)*/
			if EnrEpisodeCensoredAtDeath and DeathDt <= Enr_End then Enr_End=DeathDt;

			format EpisodeEndDt_Censor OrigEpisEndDt date9.;

	        keep Group Type PatId Enr_start Enr_End IndexDt EpisodeEndDt NumEvents FEventDt tte year EpisodeType blackoutper: IndexLook
				cens_elig cens_dth cens_dpend cens_qryend cens_episend cens_spec cens_event deathdt Event_flag;
	        run;

			*Summarize utilization overlapping only FINAL episodes;
			%ms_shaveoutside(reffile=_PtsMasterList,
	                         refstart=IndexDt,
	                         refend=EpisodeEndDt,
	                         KeepPartBf=Y,
	                         ToShaveFile=_GroupIndex,
	                         ToShaveStart=Adate,
	                         ToShaveEnd=ExpireDt,
	                         outfile=_GroupUtilFile);

			proc means data=_GroupUtilFile nway noprint;
	        var RxSup RxAmt;
	        class PatId IndexDt;
	        output out=_GroupUtilFile(rename=_freq_=NumDisp drop=_Type_) sum(RxSup RxAmt)=TotRxSup TotRxAmt;
	        run;
	
		%end;	

    	proc sort nodupkey data=_agegroups;
    	by PatId IndexDt;
    	run;

    	data _PtsMasterList;
    	merge _PtsMasterList (in=a)
    	      _GroupUtilFile (in=b)
    		  _AgeGroups;
    	by PatId IndexDt;
    	if a or b;*defensive;

		%if "&agedatecohort" = "Y" %then %do;	
			TotRxSup=0;
			TotRxAmt=0;
			NumDisp=0;	
			EPISODETYPE = 'ITT';
		%end;
    	run;

		* This will be applied later for Surveillance analysis;
		data _PtsMasterList;
		set _PtsMasterList;
		LastLookFollowed=0;
		format LastLookFollowedDt date9.;
		LastLookFollowedDt=.;
		format EndFollowUpType $20.;
		EndFollowUpType="";
		run;        

    %end;  *TYPE 1 and 2;

	****************************************;
	* Type 3 cohort definition 				;
	****************************************;
	%if %eval(&type=3) %then %do;

		*Create a master file for Censor data;
		data _PtsMasterList_Censor;
		set _PtsMasterList;
		run;

		%macro finalizeT3Master(suffix=);

	        *Keep only valid IndexDt in master file;
	      	data _PtsMasterList&suffix.;
	        merge _PtsMasterList&suffix.(in=a)
	              _Events(in=b);
	        by PatId IndexDt;
	        if a;
			
	        if first.patid then do;
	            Pt_Analysis_Episode_Num=0; 
	            Pt_Exposure_Episode_Num=0;
	        end;

	        Pt_Exposure_Episode_Num=Pt_Exposure_Episode_Num+1;

	        if &ExpPeriodStartDt. <= indexdt <= &ExpPeriodEndDt.;

	        *01: Cohort includes only the first exposure during the query period;
	        if T3CohortDef ='01' then do;
	            if first.PatId;
	        end;

			TTC=.;
			CensorDeath=0;
	        CensorEnrol=0;
	        if MaxEligtDt > ENR_END then do;
	            if EnrEpisodeCensoredAtDeath then CensorDeath=1;
	            else CensorEnrol=1;
				TTC=ENR_END-Indexdt;
	        end;
			
			*Analysis cohort should be turned off if censoring applies;
	        *Compute days at risk;
	        AnalysisCohort=0;
	        if FEventDt ne . and MinEnrolHOIMet=1 
			   and MaxEligtEnrAftDt <= ENR_END then do;
	            DAYS_FROM_EXPOS_TO_EVENT=FEventDt-IndexDt;
	            if WinOfEvent="CTRL" then DAYS_FROM_EXPOS_TO_CTRL_EVENT=DAYS_FROM_EXPOS_TO_EVENT;
	            if WinOfEvent="RISK" then DAYS_FROM_EXPOS_TO_RISK_EVENT=DAYS_FROM_EXPOS_TO_EVENT;
	            AnalysisCohort=1;
	            Pt_Analysis_Episode_Num=Pt_Analysis_Episode_Num+1;
	        end;    

			if AnalysisCohort=0 then do;
				FEventDt=.;
				WinOfEvent="";
			end; 

	        POSTENR_EXPOSURE=Enr_End-IndexDt;

	        retain Pt_Analysis_Episode_Num Pt_Exposure_Episode_Num;

	        keep Group Type PatId Enr_start Enr_End IndexDt FEventDt DAYS_FROM: year WinOfEvent MinEnrolHOIMet 
				 TTC age birth_date sex race hispanic AgeGroup Censor: AnalysisCohort Pt_: POSTENR_EXPOSURE zip: state hhs_reg cb_reg;
	        run;

		%mend finalizeT3Master;

		%finalizeT3Master(suffix=);
		%finalizeT3Master(suffix=_Censor);

        *Delete patients with multiple NDC on the incident exposure date;
        %if %str(&ExclOnSameDay)=%str(Y) %then %do; 

            data _PtsMasterList
                 _SameDayDeleted;
            merge _PtsMasterList(in=a drop=TTC)
                  _SameDayNDCs(in=b keep=PatId Rxdate rename=Rxdate=indexDt);
            by PatId IndexDt;
            if a;
            if b then output _SameDayDeleted;
            else output _PtsMasterList;
            run;

			data _PtsMasterList_Censor;                 
            merge _PtsMasterList_Censor(in=a)
                  _SameDayNDCs(in=b keep=PatId Rxdate rename=Rxdate=indexDt);
            by PatId IndexDt;
            if a and not b then output _PtsMasterList_Censor;
            run;
        %end;
		%else %do;
			data _SameDayDeleted;
			set _PtsMasterList(keep=PatId IndexDt);
			if _N_=0 then output;
			run;
		%end;


		/*Add AnalysisCohort_censor to _mstr to consolidate information to 1 file*/
		proc sql noprint undo_policy=none;
			create table _PtsMasterList as
			select x.*, y.analysiscohort as AnalysisCohort_censor
			from _PtsMasterList as x
			inner join _PtsMasterList_censor as y on x.patid = y.patid and x.indexdt = y.indexdt;
		quit;

	%end; *TYPE 3;

	****************************************;
	* Type 4 cohort definition 				;
	****************************************;
	%if %eval(&type=4) %then %do;

		data _PtsMasterList (drop=lEpisodeEndDt eligdays episode expiredt minenrolmet PrePostPriority priority:);
        set _PtsMasterList;
        by Patid IndexDt;		

		* If there are overlapping episodes, truncate the episode and set the start date  
		  at the day following the end date of the preceding episode;		
	    lEpisodeEndDt = lag(EpisodeEndDt);
	    if first.PatId then lEpisodeEndDt = .;
	    if EpisodeStartDt <= lEpisodeEndDt then do;
			EpisodeStartDt = lEpisodeEndDt + 1;
			IndexDt=EpisodeStartDt;
		end;
		FEventDt = .;
		year=year(EpisodeEndDt);
        run;

		proc sort data=_PtsMasterList;
			by patid episodeenddt;
		run;

    	data _PtsMasterList;
    	merge _PtsMasterList (in=a)
    		  _AgeGroups(rename=indexdt=EpisodeEndDt);
    	by PatId EpisodeEndDt;
    	if a;
		run;

		proc sort data=_Deliveries;
		by Patid EpisodeStartDt;
		run;

		* If there are overlapping episodes, truncate the episode and set the start date  
		  at the day following the end date of the preceding episode;	
		data _Deliveries;
		set _Deliveries;
		by Patid EpisodeStartDt;					
	    lEpisodeEndDt = lag(EpisodeEndDt);
	    if first.PatId then lEpisodeEndDt = .;
	    if EpisodeStartDt <= lEpisodeEndDt then EpisodeStartDt = lEpisodeEndDt + 1;
		drop lEpisodeEndDt;
		run;

		* Make sure the it is the correct episode start date that is in _PtsMasterList;
		proc sort data=_PtsMasterList;
		by PatId DelNum;
		run;

		proc sort data=_Deliveries;
		by PatId DelNum;
		run;

		data _PtsMasterList;
		merge _PtsMasterList(in=a drop=EpisodeStartDt)
			  _Deliveries (in=b keep=PatId DelNum EpisodeStartDt);
		by Patid DelNum;
		if a;
		IndexDt=EpisodeStartDt;
		Duration=EpisodeEndDt - EpisodeStartDt;
	    Has3Trim=0;
	    if Duration>=181 then Has3Trim=1;
		format PrePostInd $10.;

		PrePostInd="NONE";
		if HadNone ne 1 then do;
			if duration <=258 then PrePostInd = "PRETERM";
			else if 259 <= duration <=280 then PrePostInd = "TERM";
			else if duration >=281 then PrePostInd = "POSTTERM";
		end;

		*pre/post icd-10 strata;
		if EpisodeEnddt < '01Oct2015'd then ICD = '09';
		else if EpisodeEnddt >= '01Oct2015'd then ICD = '10';
		run;

		proc sort data=_PtsMasterList;
			by patid encounterID;
		run;

		*select all delivery codes during index encounter;
		proc sql noprint;
			create table _deliverycodes as
			select delivery.patid label='Mother member ID',
				  /* delivery.group, */ /*To include when converting to RAF*/
				   delivery.encounterID label='Mother delivery EncounterID' ,
				   delivery.birth_type label='Birth Type',
				   delivery.codecat label='Code category',
				   delivery.codetype label='Code Type',
				   delivery.clincode label='Clinical code'
			from _GroupIndex as delivery,
				_PtsMasterList as pts
			where delivery.patid = pts.patid and 
				  delivery.encounterID = pts.encounterID;
		quit;

		*Determine birth type and number of delivery codes per encounter (Mom/Baby linkage tables);

		/*Resolving Multiple Values for the Number of Babies Per Delivery
			1.	If multiple codes within an encounter have the same birth type code, then the encounter will be assigned that codes.
			2.	If an encounter has a birth type code of 0 and any other value of 1-8, then the encounter will be assigned the non-zero code.
			3.	Only if within the encounter, there are birth type codes within the range of 1-8 and they differ, will the encounter be assigned the value of 9. */

			data _enc_birth_type1;
				set _deliverycodes;
				%do btype = 0 %to 5;
					if birth_type = &btype. then btype_&btype. = 1;
					else btype_&btype. = 0;
				%end;
				if birth_type = 8 then btype_8 = 1;
				else btype_8 = 0;
			run;

			proc means noprint nway data=_enc_birth_type1;
				var btype_: ;
				class patid encounterID;
				output out=_summarized_enc_birth_type(drop=_:) max(btype_:)=;
			run;
		
			*Merge in birth codes;
			proc sql noprint;
				create table _enc_birth_type2 as
				select x.*, 
					   y.birth_type
				from _summarized_enc_birth_type as x,
					 _deliverycodes as y
				where x.patid = y.patid and x.encounterID = y.encounterID;
			quit;

			data _enc_birth_type3;
				set _enc_birth_type2;

				total_codes = sum(of btype_0-btype_5, btype_8);
				total_nonzero_noneight = sum(of btype_1-btype_5);

				* Only 1 birth type code / only 1 non-zero/non-eight birth type code;
				if total_codes = 1 | total_nonzero_noneight = 0 then do;
					final_birth_type = birth_type;
				end;
				*one 1-5 code, and 0 and/or 8 codes - assign to 1-5. Need to recode 8 to 0 for when we take max();
				else if total_nonzero_noneight = 1 then do;
					if birth_type = 8 then final_birth_type = 0;
					else final_birth_type = birth_type;
				end;
				* Multiple birth type codes;
				else do;
					final_birth_type = 9;
				end;
		
				*assign prime number for each delivery code;
				select (birth_type);
					when (0) birth_type_primes = 2;
					when (1) birth_type_primes = 3;
					when (2) birth_type_primes = 5;
					when (3) birth_type_primes = 7;
					when (4) birth_type_primes = 11;
					when (5) birth_type_primes = 13;
					when (8) birth_type_primes = 17;					
					otherwise 	birth_type_primes = 19;
				end;
			run;

			*calculate birth type primes;
			proc sort data=_enc_birth_type3 out=primes(keep=patid encounterID birth_type_primes) nodupkey;
				by patid encounterID birth_type_primes;
			run;

			data primes1;
				set primes;
				by patid encounterID;
				retain btcomb btcombcount;
         		length btcomb 8 btcombcount 3;
				if first.encounterID then do;
		            btcomb = birth_type_primes;
		            btcombcount = 1;
		        end;
		        else do;
		            btcomb = btcomb * birth_type_primes;
		            btcombcount = btcombcount + 1;
		        end;
		        if last.encounterID then output primes1;
		    run;

			proc sql noprint;
				create table _final_birth_type as
					select x.patid
						,  x.encounterID
						,  max(x.final_birth_type) as birth_type format=3. length=3
						,  max(y.btcomb) as birth_type_primes
				from _enc_birth_type3(drop=birth_type_primes) as x, primes1 as y
				where x.patid =  y.patid and x.encounterID = y.encounterID
				group by x.patid, x.encounterID
				order by x.patid, x.encounterID;
			quit;

		*Merge birth_type/birth_type_primes with _PtsMasterList;
		data _PtsMasterList;
			merge _PtsMasterList(drop=birth_type) 
				  _final_birth_type;
			by patid encounterID;
		run;

		proc sort data=_PtsMasterList; 
			by patid delnum;
		run;

		data _Deliveries;
		merge _Deliveries(in=a)
			  _PtsMasterList (in=b keep=PatId DelNum duration);
		by PatId DelNum;
		format PrePostInd $10.;

		IndexDt=EpisodeStartDt;
		Duration=EpisodeEndDt - EpisodeStartDt;

		PrePostInd="NONE";

		if HadNone ne 1 then do;
		if . < duration <=258 then PrePostInd = "PRETERM";
		else if 259 <= duration <=280 then PrePostInd = "TERM";
		else if duration >=281 then PrePostInd = "POSTTERM";
		end;
		Eligible="N";
		if b then Eligible="Y";
		run;

		proc sort data=_PtsMasterList; 
		by patid ADate; 
		run;

		proc sort data=_GroupStartCodes nodupkey; 
		by patid ADate; 
		run;

		data _PrePostCodes;
		merge _PtsMasterList(in=a keep=patid adate delnum)              
	          _GroupStartCodes;
		by PatId ADate;
		if a;
		if not missing(PriorityGroup1) and not missing(PriorityGroup2);
		keep PatId Group PriorityGroup1 PriorityGroup2 Code DelNum;
		run;

	%end;	*TYPE 4

	****************************************;
	* Type 5 cohort definition 				;
	****************************************;
	%if %eval(&type=5) %then %do;

    	proc datasets library=work nolist nowarn;
    	delete _GroupUtilFile _PtsMasterList _dispensings;
    	quit;
	
		data _PtsMasterList;
	        set _PtsMasterList2(rename=indexdt=episodestartdt);
	        by Patid episodestartdt;
	   
	        tte =Min(EpisodeEndDt,Enr_End)-episodestartdt +1;

			StudyStartDate = &startfollowup.;
			StudyEndDate = &enddate.;

			format Group $30.;
   			Group="&ITGROUP."; 

			length episodenum episode_cens_elig episode_cens_dth episode_cens_dpend episode_cens_qryend episode_cens_episend episode_cens_spec 4;

			if first.patid then do;
				episodenum = 1;
				indexdt = episodestartdt;
			end;
			else do;
				episodenum = episodenum + 1;
				indexdt = indexdt;
			end;

			retain episodenum indexdt;

			/*Enumerate censoring criteria*/

			/*initialize censoring variables*/
			episode_cens_elig = 0;
			episode_cens_dth = 0;
			episode_cens_dpend = 0;
			episode_cens_qryend = 0;
			episode_cens_episend = 0;	
			episode_cens_spec = 0;

			/*Consider true censoring date*/
			EpisodeEndDt_Censor = min(EpisodeEndDt,Enr_End, deathdt);

			/*Max Eligibility*/
			MinEligDate = min(enr_end, deathdt, &enddate.);

			/*Censored due to enrollment*/
			If EpisodeEndDt_Censor = Enr_End then episode_cens_elig = 1; else episode_cens_elig = 0;

			/*Censored due to death*/
			If EnrEpisodeCensoredAtDeath = 1 and EpisodeEndDt_Censor = Deathdt then episode_cens_dth = 1; else episode_cens_dth = 0;

			/*Censored due to query end*/
			if EpisodeEndDt_Censor = &enddate. then episode_cens_qryend = 1; else episode_cens_qryend = 0;

			/*Censored due to DP Max*/
			%if &Censor_DPEnd. = 1 %then %do;
				if EpisodeEndDt_Censor = &dp_maxdate. then episode_cens_dpend = 1; else episode_cens_dpend = 0;

					%if &DPMax_Adjust. =1 %then %do;
						episode_Cens_QRYEND = 0; /*Reset Cens_qryend because we adjusted enddate to equal dp_maxdate*/
					%end;
			%end;

			/*Censored due to requester defined criteria*/
			if trunkdate and EpisodeEndDt_Censor = trunkdate then episode_cens_spec = 1; else episode_cens_spec = 0;

			/*Censored due to episode end date*/
				/*No adjustments have been made*/
				if (OrigEpisEndDt = EpisodeEndDt_Censor) then episode_cens_episend = 1;
				else episode_cens_episend = 0;

			/*Adjust enr_end for death*/
			if EnrEpisodeCensoredAtDeath and DeathDt <= Enr_End then Enr_End=DeathDt;

			format EpisodeEndDt_Censor OrigEpisEndDt studystartdate studyenddate indexdt MinEligDate date9.;
			label episodestartdt='Episode Start Date';

	        keep Group PatId studystartdate studyenddate indexdt episodenum episodestartdt episodeenddt episode_cens_: tte MinEligDate;
			run;

			*Summarize utilization overlapping only FINAL episodes;
			%ms_shaveoutside(reffile=_PtsMasterList,
	                         refstart=EpisodeStartdt,
	                         refend=EpisodeEndDt,
	                         KeepPartBf=Y,
	                         ToShaveFile=_GroupIndex2,
	                         ToShaveStart=Adate,
	                         ToShaveEnd=ExpireDt,
	                         outfile=_GroupUtilFile);

			proc means data=_GroupUtilFile nway noprint;
	        var RxSup;
	        class PatId EpisodeStartdt;
	        output out=_GroupUtilFile_sum(rename=_freq_=EpisodeDispensing drop=_Type_) sum(RxSup)=TotRxSup;
	    run;


		/*Merge demographics at index date*/
		proc sql noprint undo_policy=none;
			create table _PtsMasterList as
			select x.*
				, y.birth_date
				, y.sex
				, y.race
				, y.hispanic
				, y.age
				, y.agegroup
				, y.minagedate
				, y.maxagedate
				, y.zip3
				, y.state
				, y.hhs_reg
				, y.cb_reg
				, y.zip_uncertain
			from _PtsMasterList as x
			left join _agegroups as y
			on x.patid = y.patid and x.indexdt = y.indexdt
			order by x.patid , x.EpisodeStartdt DESCENDING  ;
		quit;

		proc sort data=_GroupUtilFile_sum;
    		by PatId DESCENDING EpisodeStartdt;
    	run;	

		data _PtsMasterList;
	    	merge _PtsMasterList (in=a)
	    	      _GroupUtilFile_sum (in=b);
	    	by PatId DESCENDING EpisodeStartdt;
	    	if a or b;*defensive;
		run;

		data _PtsMasterList(drop=MinEligDate LEpiStart);
			set _PtsMasterList;
			by PatId DESCENDING EpisodeStartdt;

			length gapnum Mntsfromstart 4;

			*calculate GapLength;
			if first.patid then do;
				*note: when elgibility = end of episode, gaplength = -1;
				gaplength = MinEligDate - EpisodeEndDt-1;
				gapnum = 999;
				LEpiStart = EpisodeStartdt;
			end;
			else do;
				gaplength = LEpiStart - EpisodeEndDt-1;
				LEpiStart = EpisodeStartdt;
				gapnum = EpisodeNum;
			end;
			retain LEpiStart;	

			/*Calculate months from study start*/
			Mntsfromstart = intck('month', StudyStartDate, EpisodeStartdt, 'c')+1;
    	run;

		/*Calculate cumulative time at risk and episodegap*/
		proc means data=_PtsMasterList nway noprint;
			var tte;
			class patid;
			output out=_episodelength(drop=_:) sum(tte)=EpisodeLength;
		run;

		data _PtsMasterList;
			merge _PtsMasterList
				_episodelength;
			by patid;

			year=year(indexdt);
			FEventDt = .;
		run;

		/*Finalize _dispensings*/
		proc sql noprint undo_policy=none;
			create table _dispensings as
			select distinct util.*
				, pts.indexdt
				, pts.age
				, pts.agegroup
				, pts.minagedate
				, pts.maxagedate
				, &startfollowup. as StudyStartDate format=date9.
				, &enddate. as StudyEndDate format=date9.
				, pts.episodenum
				, pts.birth_date
				, pts.sex
				, pts.race
				, pts.hispanic
				, pts.zip3
				, pts.state
				, pts.zip_uncertain
				, pts.hhs_reg
				, pts.cb_reg
				, "&ITGROUP." as Group format=$30. length 30
			from _grouputilfile as util, _ptsmasterlist as pts
			where util.patid = pts.patid and util.episodestartdt = pts.episodestartdt
			order by patid, episodenum;
		quit;

		proc means data=_dispensings noprint nway;
			var rxsup;
			class patid episodenum;
			output out = _dispensings_sum(rename=_freq_=Episodedispensing drop=_type_) sum(rxsup)=episoderxsup;
		run;

		data _dispensings;
			merge _dispensings(rename=adate=disp_rxdate rename=rxsup=disp_rxsup)
				_dispensings_sum;
			by patid episodenum;

			keep StudyStartDate StudyEndDate group patid birth_date sex race hispanic zip3 state hhs_reg cb_reg zip_uncertain agegroup minagedate maxagedate indexdt episodenum
					episodedispensing episoderxsup disp_rxdate disp_rxsup;
		run;

		/*remove labels*/
		proc datasets lib=work memtype=data nolist nowarn;
	    modify _dispensings; 
	    attrib _all_ label=' '; 
	    run;

	%end; *TYPE 5;

	*Remove all labels;
    proc datasets lib=work memtype=data nolist nowarn;
    modify _PtsMasterList; 
    attrib _all_ label=' '; 
    run;

%put NOTE: ********END OF MACRO: ms_finalizeptsmasterlist v1.4********;

%mend ms_finalizeptsmasterlist;
